#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "genetic.h"
#include "rand32.h"

int main(int argc, char * argv[]) {

  if(argc!=6) {
    fprintf(stdout,"Usage is: genetic <Np> <Ng> <Rc> <Rm> <MIF>\n");
    fprintf(stdout,"\n");
    fprintf(stdout,"Param  Description                     Bounds\n");
    fprintf(stdout,"-----  -----------------------------   ------------\n");
    fprintf(stdout," Np    Size of population of parents   [0,infinity)\n");
    fprintf(stdout," Ng    Number of generations           [0,infinity)\n");
    fprintf(stdout," Rc    Rate of Crossover               [0.0,1.0]\n");
    fprintf(stdout," Rm    Rate of Mutation                [0.0,1.0]\n");
    fprintf(stdout," MIF   Maximum Initial Fitness         [5,32]\n");
    fprintf(stdout,"\n");
    fprintf(stdout,"genetic is a demonstration of the genetic algorithm.\n");
    fprintf(stdout,"It starts with a random population of unsigned int-\n");
    fprintf(stdout,"egers, and it attempts to breed the value 0xFFFFFFFF.\n\n");
    fprintf(stdout,"The fitness of a solution is given by the number of\n");
    fprintf(stdout,"binary 1's in the solution.\n\n");
    fprintf(stdout,"Crossover: Crossover randomly selects bits from one of\n");
    fprintf(stdout,"two parents to form the bits of the offspring.\n\n");
    fprintf(stdout,"Selective Breeding: Parents are selected to breed with\n");
    fprintf(stdout,"probability proportional to their fitnesses.\n\n");
    fprintf(stdout,"Mutation: Mutation consists of randomly flipping a single\n");
    fprintf(stdout,"bit in a newly generated child.\n\n");
    fprintf(stdout,"PROCEDURE Genetic()\n");
    fprintf(stdout,"   Given Np, Ng, Rc, Rm, and MIF\n");
    fprintf(stdout,"1  Generate initial population randomly, subject to MIF\n");
    fprintf(stdout,"2  While(Current_Generation < Ng) {\n");
    fprintf(stdout,"     // Bene Gesserit Breeding Program\n");
    fprintf(stdout,"3    Breed (Rc * Np) new solutions:\n");
    fprintf(stdout,"4      Select parents with probability proportional to\n");
    fprintf(stdout,"       their fitnesses.\n");
    fprintf(stdout,"5      Perform Crossover\n");
    fprintf(stdout,"6      Mutate the offspring with probability Rm\n");
    fprintf(stdout,"     // Survival of the fittest\n");
    fprintf(stdout,"7    Kill (Rc * Np) of the least-fit solutions\n");
    fprintf(stdout,"8  EndWhile\n\n");
    fprintf(stdout,"9  return [Best member of population]\n");
    fprintf(stdout,"ENDPROECURE Genetic()\n\n");
     
    return 0;
  }

  int Np, Ng, MIF;
  double Rc, Rm;

  // parameter parsing
  Np = atoi(argv[1]);
  Ng = atoi(argv[2]);
  Rc = strtod(argv[3],NULL);
  Rm = strtod(argv[4],NULL);
  MIF = atoi(argv[5]);

  // parameter checking
  if(Np<0) {
    fprintf(stderr,"Np must be [0,infinity)\n");
    return -1;
  }

  if(Ng<0) {
    fprintf(stderr,"Ng must be [0,infinity)\n");
    return -1;
  }

  if(Rc<0.0||Rc>1.0) {
    fprintf(stderr,"Rc must be [0.0,1.0]\n");
    return -1;
  }

  if(Rm<0.0||Rm>1.0) {
    fprintf(stderr,"Rm must be [0.0,1.0]\n");
    return -1;
  }


  if(MIF<5||MIF>32) {
    fprintf(stderr,"Max. Initial. Fitness must be [5,32]\n");
    return -1;
  }

  srand(time(NULL));

  unsigned int best = genetic(Np,Ng,Rc,Rm,MIF);
  int best_fitness  = fitness(best);

  fprintf(stdout,"Best solution was 0x%08x with fitness %i\n",best,best_fitness); 
  
  return 0;
}
